"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const BbPromise = require("bluebird");
const IORedis = require("ioredis");
class RedisDriver {
    constructor(serverUrl, options = {}) {
        this.serverUrl = serverUrl;
        this.options = options;
        this.isCluster = !!options.enableClusterMode;
        if (this.isCluster) {
            const DEFAULT_OPTIONS = {
                clusterRetryStrategy(times) {
                    return 100; // retry after 100ms
                },
            };
            // if server url is master, node discovery is automatically performed
            this.client = new IORedis.Cluster([serverUrl], Object.assign({}, DEFAULT_OPTIONS, (options.ioredis || {})));
        }
        else {
            const DEFAULT_OPTIONS = {
                retryStrategy(times) {
                    return 100; // retry after 100ms
                },
            };
            this.client = new IORedis(serverUrl, Object.assign({}, DEFAULT_OPTIONS, (options.ioredis || {})));
        }
    }
    touch(key, lifetime) {
        return __awaiter(this, void 0, void 0, function* () {
            // @see https://redis.io/commands/expire#return-value
            // @type Integer reply
            //
            // 1 if the timeout was set.
            // 0 if key does not exist.
            const reply = yield this.client.expire(key, lifetime);
            return reply === 1;
        });
    }
    get(key) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this.client.get(key);
            try {
                return JSON.parse(response);
            }
            catch (e) {
                return response;
            }
        });
    }
    getMulti(keys) {
        return __awaiter(this, void 0, void 0, function* () {
            if (keys.length === 0) {
                return {};
            }
            // In cluster mode, MGET (multiple get) command requires all keys must be same key slot
            // if client does not handle this, redis will give "CROSSSLOT Keys in request don't hash to the same slot" Error
            if (this.isCluster) {
                // this is temporary fix for CROSSSLOT issue.
                // we should consider using cluster-key-slot module to optimize requests.
                // @todo optimize getMulti logic in Cluster Mode
                return yield BbPromise.reduce(keys, (hash, key) => __awaiter(this, void 0, void 0, function* () {
                    hash[key] = yield this.get(key);
                    return hash;
                }), {});
            }
            else {
                const response = yield this.client.mget(...keys);
                return keys.reduce((hash, key, index) => {
                    const val = response[index];
                    if (val !== null) {
                        try {
                            hash[key] = JSON.parse(val);
                        }
                        catch (e) {
                            hash[key] = val;
                        }
                        return hash;
                    }
                    hash[key] = undefined;
                    return hash;
                }, {});
            }
        });
    }
    set(key, value, lifetime) {
        return __awaiter(this, void 0, void 0, function* () {
            const serialized = JSON.stringify(value);
            // @see https://redis.io/commands/setex#return-value
            // @type Simple string Reply
            let reply;
            if (!lifetime) {
                reply = yield this.client.set(key, serialized);
            }
            else {
                reply = yield this.client.setex(key, lifetime, serialized);
            }
            if (reply !== "OK") {
                throw new Error(`RedisDriver failed to set: '${key} - ${value}'`);
            }
        });
    }
    del(key) {
        return __awaiter(this, void 0, void 0, function* () {
            // @see https://redis.io/commands/del#return-value
            // @type Integer reply: The number of keys that were removed.
            const reply = yield this.client.del(key);
            return reply > 0;
        });
    }
    flush() {
        return __awaiter(this, void 0, void 0, function* () {
            // @see https://redis.io/commands/flushdb
            // @type Simple string reply
            //
            // @note starting redis 4, redis support **ASYNC** flush mode.
            // since redis blocks everything during flush process,
            // flushing database which has huge number of keys can be cause redis server outage.
            // so it is great improvement, but currently AWS ElastiCache does not support redis 4.
            // @note this command does not flush entire redis database,
            // so this command **ONLY** flushes current active database
            const reply = yield this.client.flushdb();
            if (reply !== "OK") {
                throw new Error(`RedisDriver failed to flush: '${reply}'`);
            }
        });
    }
    end() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.quit(); // quit method should disconnect connection cleanly
        });
    }
}
exports.RedisDriver = RedisDriver;
//# sourceMappingURL=redis.js.map